@@ -18,14 +18,14 @@ data$alive      <- abs(data$death - 1)
 # calculate propensity weights
 
 m      <- glm(I(trt-1) ~ age + risk + severity, data=data, family=binomial(link="logit"))
-data$p <-predict(m, type = "response")
+data$p <- predict(m, type = "response")
 
-# run bandit - when leaving out P1, Propensity Bandit uses marginal prob per arm for propensities:
+# run bandit - when leaving out p, Propensity Bandit uses marginal prob per arm for propensities:
 # table(private$z)/length(private$z)
 
 f          <- alive ~ trt | age + risk + severity | p
 
-bandit     <- OfflinePropensityWeightingBandit$new(formula = f, data = data)
+bandit     <- OfflinePropensityWeightingBandit$new(formula = f, data = data, threshold = 0.01)
 
 # Define agents.
 agents      <- list(Agent$new(LinUCBDisjointOptimizedPolicy$new(0.2), bandit, "LinUCB"))
